@@ -25,9 +25,12 @@ def parse_wallet_journal(api_result):
             'amount': float(a['amount']),
             'balance': float(a['balance']),
             'reason': a['reason'],
+            # The tax fields might be an empty string, or not present
+            # at all (e.g., for corp wallet records.)  Need to handle
+            # both edge cases.
             'tax': {
-                'taxer_id': int(a['taxReceiverID'] or 0),
-                'amount': float(a['taxAmount'] or 0),
+                'taxer_id': int(a.get('taxReceiverID') or 0),
+                'amount': float(a.get('taxAmount') or 0),
             },
         }
 
